from setuptools import setup, find_packages

setup(name='redpjpj',
        version='1.0',
        description='ENSF 338 Final Project',
        author='Liam and Theodore',
        packages=find_packages(),
        install_requires=[]
        )
        